package java.awt.image;

import java.awt.color.ColorSpace;

public abstract class PackedColorModel extends ColorModel
{
    int[] maskArray;
    int[] maskOffsets;
    float[] scaleFactors;
    
    public PackedColorModel(ColorSpace colorSpace, int n, int[] array, int n2, boolean b, int n3, int n4) {
        super(n, createBitsArray(array, n2), colorSpace, n2 != 0, b, n3, n4);
        if (n < 1 || n > 32) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 32.");
        }
        this.maskArray = new int[this.numComponents];
        this.maskOffsets = new int[this.numComponents];
        this.scaleFactors = new float[this.numComponents];
        for (int i = 0; i < this.numColorComponents; ++i) {
            this.DecomposeMask(array[i], i, colorSpace.getName(i));
        }
        if (n2 != 0) {
            this.DecomposeMask(n2, this.numColorComponents, "alpha");
            if (this.nBits[this.numComponents - 1] == 1) {
                this.transparency = 2;
            }
        }
        return;
    }
    
    public PackedColorModel(ColorSpace colorSpace, int n, int n2, int n3, int n4, int n5, boolean b, int n6, int n7) {
        super(n, createBitsArray(n2, n3, n4, n5), colorSpace, n5 != 0, b, n6, n7);
        if (colorSpace.getType() != 5) {
            throw new IllegalArgumentException("ColorSpace must be TYPE_RGB.");
        }
        this.maskArray = new int[this.numComponents];
        this.maskOffsets = new int[this.numComponents];
        this.scaleFactors = new float[this.numComponents];
        this.DecomposeMask(n2, 0, "red");
        this.DecomposeMask(n3, 1, "green");
        this.DecomposeMask(n4, 2, "blue");
        if (n5 != 0) {
            this.DecomposeMask(n5, 3, "alpha");
            if (this.nBits[3] == 1) {
                this.transparency = 2;
            }
        }
        return;
    }
    
    public final int getMask(int n) {
        return this.maskArray[n];
    }
    
    public final int[] getMasks() {
        return (int[])(int[])this.maskArray.clone();
    }
    
    private void DecomposeMask(int n, int n2, String str) {
        int n3 = 0;
        int n4 = this.nBits[n2];
        this.maskArray[n2] = n;
        if (n != 0) {
            while ((n & 0x1) == 0x0) {
                n >>>= 1;
                ++n3;
            }
        }
        if (n3 + n4 > this.pixel_bits) {
            throw new IllegalArgumentException(str + " mask " + Integer.toHexString(this.maskArray[n2]) + " overflows pixel (expecting " + this.pixel_bits + " bits");
        }
        this.maskOffsets[n2] = n3;
        if (n4 == 0) {
            this.scaleFactors[n2] = 256.0f;
        }
        else {
            this.scaleFactors[n2] = 255.0f / (float)((1 << n4) - 1);
        }
        return;
    }
    
    @Override
    public SampleModel createCompatibleSampleModel(int n, int n2) {
        return new SinglePixelPackedSampleModel(this.transferType, n, n2, this.maskArray);
    }
    
    @Override
    public boolean isCompatibleSampleModel(SampleModel sampleModel) {
        if (!(sampleModel instanceof SinglePixelPackedSampleModel)) {
            return false;
        }
        if (this.numComponents != sampleModel.getNumBands()) {
            return false;
        }
        if (sampleModel.getTransferType() != this.transferType) {
            return false;
        }
        int[] bitMasks = ((SinglePixelPackedSampleModel)sampleModel).getBitMasks();
        if (bitMasks.length != this.maskArray.length) {
            return false;
        }
        int n = (int)((1L << DataBuffer.getDataTypeSize(this.transferType)) - 1L);
        int i = 0;
        while (i < bitMasks.length) {
            if ((n & bitMasks[i]) != (n & this.maskArray[i])) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public WritableRaster getAlphaRaster(WritableRaster writableRaster) {
        if (!this.hasAlpha()) {
            return null;
        }
        int minX = writableRaster.getMinX();
        int minY = writableRaster.getMinY();
        return writableRaster.createWritableChild(minX, minY, writableRaster.getWidth(), writableRaster.getHeight(), minX, minY, new int[] { writableRaster.getNumBands() - 1 });
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof PackedColorModel)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }
        PackedColorModel packedColorModel = (PackedColorModel)o;
        int numComponents = packedColorModel.getNumComponents();
        if (numComponents != this.numComponents) {
            return false;
        }
        int i = 0;
        while (i < numComponents) {
            if (this.maskArray[i] != packedColorModel.getMask(i)) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private static final int[] createBitsArray(int[] array, int i) {
        int length = array.length;
        int[] array2 = new int[length + ((i != 0) ? 1 : 0)];
        int j = 0;
        while (j < length) {
            array2[j] = countBits(array[j]);
            if (array2[j] < 0) {
                throw new IllegalArgumentException("Noncontiguous color mask (" + Integer.toHexString(array[j]) + "at index " + j);
            }
            ++j;
        }
        if (i != 0) {
            array2[length] = countBits(i);
            if (array2[length] < 0) {
                throw new IllegalArgumentException("Noncontiguous alpha mask (" + Integer.toHexString(i));
            }
        }
        return array2;
    }
    
    private static final int[] createBitsArray(int i, int j, int k, int l) {
        int[] array = new int[3 + ((l == 0) ? 0 : 1)];
        array[0] = countBits(i);
        array[1] = countBits(j);
        array[2] = countBits(k);
        if (array[0] < 0) {
            throw new IllegalArgumentException("Noncontiguous red mask (" + Integer.toHexString(i));
        }
        if (array[1] < 0) {
            throw new IllegalArgumentException("Noncontiguous green mask (" + Integer.toHexString(j));
        }
        if (array[2] < 0) {
            throw new IllegalArgumentException("Noncontiguous blue mask (" + Integer.toHexString(k));
        }
        if (l != 0) {
            array[3] = countBits(l);
            if (array[3] < 0) {
                throw new IllegalArgumentException("Noncontiguous alpha mask (" + Integer.toHexString(l));
            }
        }
        return array;
    }
    
    private static final int countBits(int n) {
        int n2 = 0;
        if (n != 0) {
            while ((n & 0x1) == 0x0) {
                n >>>= 1;
            }
            while ((n & 0x1) == 0x1) {
                n >>>= 1;
                ++n2;
            }
        }
        if (n != 0) {
            return -1;
        }
        return n2;
    }
}
